@@ -28,7 +28,7 @@ ppgasp <- function(design, response,trend=matrix(1,dim(response)[1],1),zero.mean
     stop("one cannot fix and estimate the nugget at the same time \n")  
   }
   
-  if(!is.na(range.par)){  
+  if(!is.na(range.par)[1]){  
     if(length(range.par)!=dim(as.matrix(design))[2]){
       stop("range.par should either be fixed or estimated.")    
     }
@@ -180,7 +180,7 @@ ppgasp <- function(design, response,trend=matrix(1,dim(response)[1],1),zero.mean
   ####################correlation matrix
   
   model@nugget.est <- nugget.est
-  if(class(R0)=='logical'){
+  if(is.na(R0)[1]){ ##no given value
     if(!model@isotropic){
       model@R0 = as.list(1:model@p)
       for(i in 1:model@p){
@@ -198,10 +198,10 @@ ppgasp <- function(design, response,trend=matrix(1,dim(response)[1],1),zero.mean
         model@R0[[1]]=euclidean_distance(model@input,model@input)
       }
     }
-  }else if(class(R0)=='matrix'){
+  }else if(class(R0)[1]=='matrix'){ ##if given a matrix
     model@R0 = as.list(1)
     model@R0[[1]]=R0
-  }else if(class(R0)=='list'){
+  }else if(class(R0)[1]=='list'){ ##if given a matrix
     model@R0=R0
   }else{
     stop("R0 should be either a matrix or a list \n")
@@ -231,7 +231,7 @@ ppgasp <- function(design, response,trend=matrix(1,dim(response)[1],1),zero.mean
   }
   
   
-  if(is.na(range.par)){
+  if(is.na(range.par)[1]){
     ########this also depends on the kernel
     ##get some initial values of beta that we can start from a good set of range parameters to optimize
     
@@ -357,7 +357,7 @@ ppgasp <- function(design, response,trend=matrix(1,dim(response)[1],1),zero.mean
           #  stop(tt_all)
           #}
   
-          if(class(tt_all)!="try-error"){
+          if(class(tt_all)[1]!="try-error"){
             if(model@nugget.est==F){
               nugget_par=nugget
             }else{
@@ -424,7 +424,7 @@ ppgasp <- function(design, response,trend=matrix(1,dim(response)[1],1),zero.mean
           
         }
 
-        if(class(tt_all)!="try-error"){
+        if(class(tt_all)[1]!="try-error"){
           if(model@nugget.est==F){
             nugget_par=nugget
           }else{
@@ -493,7 +493,7 @@ ppgasp <- function(design, response,trend=matrix(1,dim(response)[1],1),zero.mean
           
         }
         
-        if(class(tt_all)!="try-error"){
+        if(class(tt_all)[1]!="try-error"){
           # if(model@nugget.est==F){
           #   nugget_par=nugget
           # }else{
